@@ -1,4 +1,4 @@
-/* $Id: gdict-pref-dialog.c,v 1.40 2004/02/09 08:37:23 pborelli Exp $ */
+/* $Id: gdict-pref-dialog.c,v 1.41 2004/05/08 01:57:09 gman Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -238,11 +238,9 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
         add_atk_namedesc(GTK_WIDGET(button), NULL, _("Reset port to default"));
         add_atk_namedesc(server_label, _("Server"), _("Server"));
         add_atk_namedesc( GTK_WIDGET (pref_dialog->server_entry), _("Server Entry"), _("Enter the Server Name"));
-        add_atk_relation( GTK_WIDGET (pref_dialog->server_entry), server_label, ATK_RELATION_LABELLED_BY);
 
         add_atk_namedesc(port_label, _("Port"), _("Port"));
         add_atk_namedesc( GTK_WIDGET(pref_dialog->port_entry), _("Port Entry"), _("Enter the Port Number"));
-        add_atk_relation( GTK_WIDGET(pref_dialog->port_entry), port_label, ATK_RELATION_LABELLED_BY);
     }
 
     if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/port", NULL)) {
