@@ -26,9 +26,9 @@ struct floppy_struct param;
 extern int errno;
 
 void
-fd_print (GFloppy *floppy, MessageType type, gchar *string)
+fd_print (GFloppy *floppy, MessageType type, char *string)
 {
-	gchar *buf;
+	char *buf;
 
 	buf = g_strdup_printf ("%c%s", type, string);
 
@@ -44,7 +44,7 @@ format_disk (GFloppy *floppy, gint ctrl)
 {
 	struct format_descr descr;
 	int track;
-	gchar *msg;
+	char *msg;
 
 	fd_print (floppy, MSG_MESSAGE, _("Formatting the disk..."));
 
@@ -93,7 +93,7 @@ verify_disk (GFloppy *floppy)
 {
 	unsigned char *data;
 	int fd, cyl_size, cyl, count;
-	gchar *msg;
+	char *msg;
 
 	cyl_size = param.sect*param.head*512;
 	data = (unsigned char *) g_malloc (cyl_size);
@@ -176,7 +176,7 @@ gint
 fdformat_disk (GFloppy *floppy)
 {
 	gint ctrl;
-	gchar *msg;
+	char *msg;
 
 	if (access(floppy->device,W_OK) < 0) {
 		msg = g_strdup_printf (_("Unable to write to device %s"), floppy->device);
